#AutoIt3Wrapper_Au3Check_Parameters=-d -w 1 -w 2 -w 3 -w 4 -w 5 -w 6
#include-once
#include "Bass.au3"

; #INDEX# =======================================================================================================================
; Title .........: BassExt.au3 BETA
; Description ...: Extended functions for bass.au3
;                  Callback routines, Peak & RMS Level & more to come
; Author ........: Eukalyptus, Prog@ndy
; ===============================================================================================================================

; #CURRENT# =====================================================================================================================
;			_BASS_EXT_Startup
;           _BASS_EXT_CreateStreamStruct
;           _BASS_EXT_AsioLinkInOut
;           _BASS_EXT_ChannelGetRMSLevel
;           _BASS_EXT_ResetPeakLevel
;           _BASS_EXT_Level2dB
;           _BASS_EXT_dB2Level
; #INTERNAL_USE_ONLY#============================================================================================================
;			__BASS_EXT_GetCallBackPointer()
; ===============================================================================================================================

;Global $BASS_EXT_AsioNotifyProc = 0
;Global $BASS_EXT_SyncProc = 0
;Global $BASS_EXT_EncodeNotifyProc = 0
;Global $BASS_EXT_FileSeekProc = 0
;Global $BASS_EXT_FileReadProc = 0
;Global $BASS_EXT_FileLenProc = 0
;Global $BASS_EXT_FileCloseProc = 0
Global $BASS_EXT_StreamProc = 0
Global $BASS_EXT_RecordProc = 0
Global $BASS_EXT_EncodeProc = 0
Global $BASS_EXT_DSPProc = 0
Global $BASS_EXT_DownloadProc = 0
Global $BASS_EXT_AsioProc = 0
Global $BASS_EXT_DspPeakProc = 0

Global Const $BASS_EXT_StructPeak = "double;double;double;double"

Global Const $BASS_EXT_STREAMPROC_DUMMY = 0
Global Const $BASS_EXT_STREAMPROC_PUSH = -1
Global Const $BASS_EXT_StreamPutFileData = 1

Global $_ghBassEXTDll = -1



Func _BASS_EXT_ChannelGetPhaseData($handle, $samples)
	If $samples < 32 Then $samples = 32
	If $samples > 2048 Then $samples = 2048
	Local $tStruct=DllStructCreate("float[" & $samples+1 & "];float[" & $samples+1 & "]")
	Local $aReturn[$samples+2][2]
	Local $bass_ext_ret = DllCall($_ghBassEXTDll, "dword", "_BASS_EXT_ChannelGetPhaseData", "dword", $handle, "dword", $samples, "ptr", DllStructGetPtr($tStruct))
	If @error Then
		Return SetError(@error, 0, $aReturn)
	Else
		For $i=1 To $samples+1
			$aReturn[$i-1][0]=DllStructGetData($tStruct,1,$i)
			$aReturn[$i-1][1]=DllStructGetData($tStruct,2,$i)
		Next
		;$aReturn[0][0]=$samples+1
		Return SetError($bass_ext_ret[0]=False, 0, $aReturn)
	EndIf
EndFunc



; #FUNCTION# ====================================================================================================================
; Name...........: _BASS_EXT_Startup
; Description ...: Starts up BASS functions.
; Syntax.........: _BASS_EXT_Startup($sBassEXTDLL)
; Parameters ....:  -	$sBassDLL	-	The relative path to BassEXT.dll.
; Return values .: Success      - Returns True
;                  Failure      - Returns False and sets @ERROR
;									@error will be set to-
;										- $BASS_ERR_DLL_NO_EXIST	-	File could not be found.
; Author ........: Prog@ndy
; Modified.......: Brett Francis (BrettF), Eukalyptus
; Remarks .......:
; Related .......:
; Link ..........;
; Example .......;
; ===============================================================================================================================
Func _BASS_EXT_Startup($sBassEXTDLL = "BassExt.dll")
	If $_ghBassEXTDll <> -1 Then Return True
	If Not FileExists($sBassEXTDLL) Then Return SetError($BASS_ERR_DLL_NO_EXIST, 0, False)
	$_ghBassEXTDll = DllOpen($sBassEXTDLL)
	If Not @error Then ; Get Pointer to the callback functions
		$BASS_EXT_StreamProc = __BASS_EXT_GetCallBackPointer(1)
		If @error Then Return SetError(1, 1, 0)
		$BASS_EXT_RecordProc = __BASS_EXT_GetCallBackPointer(2)
		If @error Then Return SetError(1, 2, 0)
		$BASS_EXT_EncodeProc = __BASS_EXT_GetCallBackPointer(3)
		If @error Then Return SetError(1, 3, 0)
		$BASS_EXT_DSPProc = __BASS_EXT_GetCallBackPointer(4)
		If @error Then Return SetError(1, 4, 0)
		$BASS_EXT_DownloadProc = __BASS_EXT_GetCallBackPointer(5)
		If @error Then Return SetError(1, 5, 0)
		$BASS_EXT_AsioProc = __BASS_EXT_GetCallBackPointer(6)
		If @error Then Return SetError(1, 6, 0)
		$BASS_EXT_DspPeakProc = __BASS_EXT_GetCallBackPointer(7)
		If @error Then Return SetError(1, 7, 0)
	EndIf
	Return $_ghBassEXTDll <> -1
EndFunc   ;==>_BASS_EXT_Startup

; #FUNCTION# ====================================================================================================================
; Name...........: Func _BASS_EXT_CreateStreamStruct()
; Description ...: Set a Stream to use in a callbackfuntion
; Syntax.........: Func _BASS_EXT_CreateStreamStruct($hStream, $iType=1)
; Parameters ....:  -$hStream 			-	The handle of the stream
;                   -$iType             -   The type of the stream
;                                           - $BASS_EXT_STREAMPROC_DUMMY: 	Stream is STREAMPROC_DUMMY and the sampledata is feed through; OR
;                                                							Stream is a "normal" stream and the sampledata is received from it (only AsioProc Input and StreamProc)
;                                           - $BASS_EXT_STREAMPROC_PUSH: 	Stream is STREAMPROC_PUSH and the sampledata is pushed to it by BASS_StreamPutData
;                                           - $BASS_EXT_StreamPutFileData: 	Stream was created by BASS_StreamCreateFileUser and sampledata is pushed to it by BASS_StreamPutFileData
;
; Return values .: Success      - Returns If successful, then a struct to pass as userdata is returned
;                  Failure      - Returns 0
; Author ........: Eukalyptus
; Modified.......:
; Remarks .......:
; Related .......: BASS_ChannelGetData, BASS_StreamPutData, BASS_StreamPutFileData
; Link ..........;
; Example .......;
; ===============================================================================================================================
Func _BASS_EXT_CreateStreamStruct($hStream, $iType = 0)
	If Not $hStream Then Return SetError(1, 0, 0)
	Local $tStruct = DllStructCreate("dword;dword")
	Switch $iType
		Case $BASS_EXT_STREAMPROC_DUMMY
			DllStructSetData($tStruct, 1, 1)
		Case $BASS_EXT_STREAMPROC_PUSH
			DllStructSetData($tStruct, 1, 2)
		Case $BASS_EXT_StreamPutFileData
			DllStructSetData($tStruct, 1, 3)
		Case Else
			DllStructSetData($tStruct, 1, 1);?
	EndSwitch
	DllStructSetData($tStruct, 2, $hStream)
	Return $tStruct
EndFunc   ;==>_BASS_EXT_CreateStreamStruct

; #FUNCTION# ====================================================================================================================
; Name...........: Func _BASS_EXT_AsioLinkInOut()
; Description ...: Links an inputchannel to an outputchannel
; Syntax.........: Func _BASS_EXT_AsioLinkInOut($inputchannel, $outputchannel)
; Parameters ....:  -	$inputchannel 			-	The input channel number... 0 = first.
;					-	$outputchannel 			-	The output channel number... 0 = first.
; Return values .: Success      - Returns If successful, then TRUE is returned, else FALSE is returned. Use
;                  Failure      - Returns 0
; Author ........: Eukalyptus
; Modified.......:
; Remarks .......: For default input 0 is linked to output 0, input 1 to outout 1 and so on...
; Related .......:
; Link ..........;
; Example .......;
; ===============================================================================================================================
Func _BASS_EXT_AsioLinkInOut($inputchannel, $outputchannel)
	Local $bass_asio_ret = DllCall($_ghBassEXTDll, "dword", "_BASS_EXT_AsioLinkInOut", "dword", $inputchannel, "dword", $outputchannel)
	If @error Then
		Return SetError(@error, "", 0)
	Else
		Return SetError(0, "", $bass_asio_ret[0])
	EndIf
EndFunc   ;==>_BASS_EXT_AsioLinkInOut

; #FUNCTION# ====================================================================================================================
; Name...........: _BASS_EXT_ChannelGetRMSLevel
; Description ...: Retrieves the level (RMS/VU amplitude) of a stream, MOD music or recording channel.
; Syntax.........: _BASS_EXT_ChannelGetRMSLevel($handle, $window)
; Parameters ....:  -	$handle		-	Handle The channel handle...
;										-	HCHANNEL, HMUSIC, HSTREAM, HRECORD or HSAMPLE handles accepted.
;                   -   $window     -   time window to calculate the average RMS
; Return values .: Success      - Returns the amplitude.
;									- Level of the left channel is returned in the low word (low 16-bits)
;									- Level of the right channel is returned in the high word (high 16-bits)
;									- If the channel is mono, then the low word is duplicated in the high word.
;									- The level ranges linearly from 0 (silent) to 32768 (max)
;                  Failure      - Returns 0
; Author ........: Eukalyptus
; Modified.......:
; Remarks .......:
; Related .......:
; Link ..........;
; Example .......;
; ===============================================================================================================================
Func _BASS_EXT_ChannelGetRMSLevel($handle, $window = 0.03)
	Local $bass_ext_ret = DllCall($_ghBassEXTDll, "DWORD", "_BASS_EXT_ChannelGetRMSLevel", "DWORD", $handle, "float", $window)
	If @error Then Return SetError(1, 1, 0)
	;If $bass_ext_ret[0] = $BASS_DWORD_ERR Then Return SetError(_BASS_ErrorGetCode(), 0, 0)
	Return $bass_ext_ret[0]
EndFunc   ;==>_BASS_EXT_ChannelGetRMSLevel

; #FUNCTION# ====================================================================================================================
; Name...........: Func _BASS_EXT_Level2dB()
; Description ...: Returns level as dB
; Syntax.........: Func _BASS_EXT_Level2dB($level, $mode = 0)
; Parameters ....:  -	$level  				-	linear level as returned by _BASS_ChannelGetLevel()
;					-	$mode                       0    : returns dB [-1024 to 0]
;                                                   else : returns db [0 to 1]
;                                                          $mode defines lowest dB value:
;                                                          for example: $mode=30: value lower -30db = zero, value greater -30dB = [0 to 1]
;                                                                       $mode=60: value lower -60db = zero, value greater -60dB = [0 to 1]
; Return values .: Success      - If successful, then the level as dB is returned
;                  Failure      - Returns 0
; Author ........: Eukalyptus
; Modified.......:
; Remarks .......: if $mode is set, all values lower -$mode dB are lost; use only to draw a levelmeter
; Related .......:
; Link ..........;
; Example .......;
; ===============================================================================================================================
Func _BASS_EXT_Level2dB($level, $mode = 0)
	If $level <= 0 Then Return -1024
	If $level > 1 Then $level /= 32768
	Local $Return = 20 * Log($level) / Log(10)
	If $Return < -1024 Then $Return = -1024
	If $Return > 0 Then $Return = 0
	If $mode Then
		$Return += $mode
		If $Return < 0 Then $Return = 0
		$Return /= $mode
		If $Return < 0 Then $Return = 0
		If $Return > 1 Then $Return = 1
	EndIf
	Return $Return
EndFunc   ;==>_BASS_EXT_Level2dB

; #FUNCTION# ====================================================================================================================
; Name...........: Func _BASS_EXT_dB2Level()
; Description ...: Returns dB as linear level
; Syntax.........: Func _BASS_EXT_dB2Level($dB, $mode = 0)
; Parameters ....:  -	$dB  				    -	decibel level as returned by _BASS_EXT_Level2dB / $mode 0
; Return values .: Success      - If successful, then the linear level is returned as floating point value
;                  Failure      - Returns 0
; Author ........: Eukalyptus
; Modified.......:
; Remarks .......:
; Related .......:
; Link ..........;
; Example .......;
; ===============================================================================================================================
Func _BASS_EXT_dB2Level($dB)
	Return 10 ^ ($dB / 20)
EndFunc   ;==>_BASS_EXT_dB2Level

; #FUNCTION# ====================================================================================================================
; Name...........: Func _BASS_EXT_ResetPeakLevel()
; Description ...: Resets max peak berfore next buffer update
; Syntax.........: Func _BASS_EXT_ResetPeakLevel($tStruct, $flag=2)
; Parameters ....:  -	$tStruct        -   decibel level as returned by _BASS_EXT_Level2dB / $mode 0
;                   -   $flag           -   0: reset left channel
;                                       -   1: reset right channel
;                                       -   2: reset both
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Eukalyptus
; Modified.......:
; Remarks .......:
; Related .......:
; Link ..........;
; Example .......;
; ===============================================================================================================================
Func _BASS_EXT_ResetPeakLevel($tStruct, $flag = 2)
	Switch $flag
		Case 0
			DllStructSetData($tStruct, 3, 2)
			If @error Then Return SetError(@error, 0, False)
		Case 1
			DllStructSetData($tStruct, 4, 2)
			If @error Then Return SetError(@error, 0, False)
		Case Else
			DllStructSetData($tStruct, 3, 2)
			If @error Then Return SetError(@error, 0, False)
			DllStructSetData($tStruct, 4, 2)
			If @error Then Return SetError(@error, 0, False)
	EndSwitch
	Return SetError(0, 0, True)
EndFunc   ;==>_BASS_EXT_ResetPeakLevel


; #FUNCTION# ====================================================================================================================
; Name...........: __BASS_EXT_GetCallBackPointer
; Author ........: Eukalyptus
; ===============================================================================================================================
Func __BASS_EXT_GetCallBackPointer($iCBFunc = 0)
	Switch $iCBFunc
		Case 1 To 7
			Local $aResult = DllCall($_ghBassEXTDll, "ptr", "_BASS_EXT_GetCallBackPointer", "dword", $iCBFunc)
			If @error Then Return SetError(1, 0, 0)
			Return $aResult[0]
	EndSwitch
	Return 0
EndFunc   ;==>__BASS_EXT_GetCallBackPointer

; #SignalFlow# ===================================================================================================================
; Signalflow description
; ===============================================================================================================================
; AsioProc:
;
;
;
;